Loading...
ppc/sys/fork.s Libc-320 Libc-391
--- Libc/Libc-320/ppc/sys/fork.s
+++ Libc/Libc-391/ppc/sys/fork.s
@@ -1,9 +1,7 @@
 /*
- * Copyright (c) 1999 Apple Computer, Inc. All rights reserved.
+ * Copyright (c) 1999-2004 Apple Computer, Inc. All rights reserved.
  *
  * @APPLE_LICENSE_HEADER_START@
- * 
- * Copyright (c) 1999-2003 Apple Computer, Inc.  All Rights Reserved.
  * 
  * This file contains Original Code and/or Modifications of Original Code
  * as defined in and that are subject to the Apple Public Source License
@@ -34,134 +32,160 @@
  *	Created from M68K sources
  */
 
-#import <sys/syscall.h>
-#import <architecture/ppc/asm_help.h>
-#import	<architecture/ppc/pseudo_inst.h>
-#import	<mach/ppc/syscall_sw.h>
+/* We use mode-independent "g" opcodes such as "srgi".  These expand
+ * into word operations when targeting __ppc__, and into doubleword
+ * operations when targeting __ppc64__.
+ */
+#include <architecture/ppc/mode_independent_asm.h>
 
-/* We use 8 bytes for LOCAL_VAR(1) and LOCAL_VAR(2) */
-NESTED(_fork, 8, 0, 0, 0)
-	CALL_EXTERN(__cthread_fork_prepare)
+#include "SYS.h"
+
+
+MI_ENTRY_POINT(_fork)
+    MI_PUSH_STACK_FRAME
+    
+    MI_CALL_EXTERNAL(__cthread_fork_prepare)
+    
 #if defined(__DYNAMIC__)
-.cstring
+    .cstring
 LC1:
 	.ascii	"__dyld_fork_prepare\0"
-.text
+    .text
 	.align 2
 	mflr	r0
-	bl	1f
+	bcl     20,31,1f
 1:	mflr	r3
 	mtlr	r0
 	addis	r3,r3,ha16(LC1-1b)
 	addi	r3,r3,lo16(LC1-1b)
-	addi 	r4,r1,LOCAL_VAR(1)
-	bl 	__dyld_func_lookup
-	lwz	r3,LOCAL_VAR(1)(r1)
+	addi 	r4,r1,SF_LOCAL1
+	bl      __dyld_func_lookup
+	lg      r3,SF_LOCAL1(r1)
 	mtspr 	ctr,r3
 	bctrl	
 #endif
-	li	r0,SYS_fork
-	sc
-	b	Lbotch			// error return
 
-	cmpwi	r4,0
-	beq	Lparent			// parent, since a1 == 0 in parent,
-					//		       1 in child
+	li      r0,SYS_fork
+	sc                      // do the fork
+	b       Lbotch			// error return
+
+	cmpwi	r4,0            // parent (r4==0) or child (r4==1) ?
+	beq     Lparent         // parent, since r4==0
+
+                            
+/* Here if we are the child.  */
+
 #if defined(__DYNAMIC__)
-.cstring
+    .cstring
 LC3:
 	.ascii	"__dyld_fork_child\0"
-.text
+    .text
 	.align 2
 	mflr	r0
-	bl	1f
+	bcl     20,31,1f
 1:	mflr	r3
 	mtlr	r0
 	addis	r3,r3,ha16(LC3-1b)
 	addi	r3,r3,lo16(LC3-1b)
-	addi 	r4,r1,LOCAL_VAR(1)
-	bl 	__dyld_func_lookup
-	lwz	r3,LOCAL_VAR(1)(r1)
+	addi 	r4,r1,SF_LOCAL1
+	bl      __dyld_func_lookup
+	lg      r3,SF_LOCAL1(r1)
 	mtspr 	ctr,r3
 	bctrl	
 #endif
-	li	r3,0x0			// clear cached pid in child
-	REG_TO_EXTERN(r3,__current_pid)
-	CALL_EXTERN(__cthread_fork_child)
+
+    li      r9,0
+    MI_GET_ADDRESS(r8,__current_pid)
+    stw     r9,0(r8)            // clear cached pid in child
+    
+	MI_CALL_EXTERNAL(__cthread_fork_child)
+    
 #if defined(__DYNAMIC__)
-.cstring
+    .cstring
 LC4:
 	.ascii	"__dyld_fork_child_final\0"
-.text
+    .text
 	.align 2
 	mflr	r0
-	bl	1f
+	bcl     20,31,1f
 1:	mflr	r3
 	mtlr	r0
 	addis	r3,r3,ha16(LC4-1b)
 	addi	r3,r3,lo16(LC4-1b)
-	addi 	r4,r1,LOCAL_VAR(1)
-	bl 	__dyld_func_lookup
-	lwz	r3,LOCAL_VAR(1)(r1)
+	addi 	r4,r1,SF_LOCAL1
+	bl      __dyld_func_lookup
+	lg      r3,SF_LOCAL1(r1)
 	mtspr 	ctr,r3
 	bctrl	
 #endif
 
-	li	r3,0
+	li	r3,0        // flag for "we are the child"
 	b	Lreturn
 
+
+/* Here if we are the parent, with:
+ *  r3 = child's pid
+ */
 Lparent:
+	stg     r3,SF_LOCAL2(r1)	// save child pid in stack
+    
 #if defined(__DYNAMIC__)
-	stw	r3,LOCAL_VAR(2)(r1)	// save child pid
 	mflr	r0
-	bl	1f
+	bcl     20,31,1f
 1:	mflr	r3
 	mtlr	r0
 	addis	r3,r3,ha16(LC2-1b)
 	addi	r3,r3,lo16(LC2-1b)
-	addi 	r4,r1,LOCAL_VAR(1)
-	bl 	__dyld_func_lookup
-	lwz	r3,LOCAL_VAR(1)(r1)
+	addi 	r4,r1,SF_LOCAL1
+	bl      __dyld_func_lookup
+	lg      r3,SF_LOCAL1(r1)
 	mtspr 	ctr,r3
 	bctrl		
 #endif
-	CALL_EXTERN(__cthread_fork_parent)
-#if defined(__DYNAMIC__)
-	lwz	r3,LOCAL_VAR(2)(r1)
-#endif
-	b	Lreturn
+    
+    b       Lparent_return      // clean up and return child's pid
+
+
+/* Here if the fork() syscall failed.  We're still the parent.  */
 
 Lbotch:	
+
 #if defined(__DYNAMIC__)
-.cstring
+    .cstring
 LC2:
 	.ascii	"__dyld_fork_parent\0"
-.text
+    .text
 	.align 2
-	stw	r3,LOCAL_VAR(2)(r1)	// save error value
+	stg     r3,SF_LOCAL2(r1)	// save error return in stack
 	mflr	r0
-	bl	1f
+	bcl     20,31,1f
 1:	mflr	r3
 	mtlr	r0
 	addis	r3,r3,ha16(LC2-1b)
 	addi	r3,r3,lo16(LC2-1b)
-	addi 	r4,r1,LOCAL_VAR(1)
-	bl 	__dyld_func_lookup
-	lwz	r3,LOCAL_VAR(1)(r1)
+	addi 	r4,r1,SF_LOCAL1
+	bl      __dyld_func_lookup
+	lg      r3,SF_LOCAL1(r1)
 	mtspr 	ctr,r3
 	bctrl
-	lwz	r3,LOCAL_VAR(2)(r1)	// restore error value for cerror
-		
+	lg      r3,SF_LOCAL2(r1)    // restore error code
 #endif
-	CALL_EXTERN(cerror)
+
+	MI_CALL_EXTERNAL(cerror)
+    li      r3,-1               // get an error return code
+	stg     r3,SF_LOCAL2(r1)	// save return code in stack
+    
 	/*
 	 * We use cthread_fork_parent() to clean up after a fork error
 	 * (unlock cthreads and mailloc packages) so the parent
 	 * process can Malloc() after fork() errors without
 	 * deadlocking.
 	 */
-	CALL_EXTERN_AGAIN(__cthread_fork_parent)
-	li32	r3,-1			// error return
-Lreturn:	RETURN
-END(_fork)
+     
+Lparent_return:
+	MI_CALL_EXTERNAL(__cthread_fork_parent)
+	lg      r3,SF_LOCAL2(r1)    // return -1 on error, child's pid on success
+    
+Lreturn:
+    MI_POP_STACK_FRAME_AND_RETURN